﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;
using MacGruber;

namespace HAL9001
{
    public class EnabledRelay : MVRScript
    {
		protected EventTrigger     m_eventTrigger;

		protected JSONStorableBool m_rememberTrigger;

		protected bool             m_triggerActive;


        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>EnabledRelay</b></size>\nLogicBrick that triggers directly after <i>Trigger</i> is called, but only when the containing Atom or the script is enabled/on. Useful if you want to delay or prevent a trigger.</color>",
				210.0f, true
			);
			
			m_eventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", m_eventTrigger.OpenPanel, false);

			m_rememberTrigger = Utils.SetupToggle(this, "Remember Trigger", false, false);

			Utils.SetupButton(this, "Test Trigger", Trigger, false);
			Utils.SetupAction(this, "Trigger", Trigger);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();

			m_triggerActive = false;
		}

		private void OnAtomRename(string oldid, string newid)
		{
			m_eventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[m_eventTrigger.Name]  = m_eventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				m_eventTrigger.RestoreFromJSON( jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			m_eventTrigger.Remove();
		}
		
		private void Update()
		{
			m_eventTrigger.Update();
			ProcessTrigger();
		}

		private void Trigger()
		{
			m_triggerActive = true;
			ProcessTrigger();
		}

		private void ProcessTrigger()
		{
			if (m_triggerActive)
			{
				if (this.gameObject.activeInHierarchy && this.enabled)
				{
					m_eventTrigger.Trigger();
					m_triggerActive = false;
				}
				else if (!m_rememberTrigger.val)
				{
					// object not active, and trigger not "remembered" > forget about it
					m_triggerActive = false;
				}
			}
		}
    }
}
