using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;
using MacGruber;

namespace HAL9001
{
    public class PerlinNoiseMovement : MVRScript
    {
		protected JSONStorableFloat m_fScale;
		protected JSONStorableFloat m_fSpeed;

		protected JSONStorableFloat m_fDeltaPosX;
		protected JSONStorableFloat m_fDeltaPosY;
		protected JSONStorableFloat m_fDeltaPosZ;

		protected JSONStorableFloat m_fDeltaRotX;
		protected JSONStorableFloat m_fDeltaRotY;
		protected JSONStorableFloat m_fDeltaRotZ;

        protected float      m_fTime;


        public override void Init()
		{
			SimpleTriggerHandler.LoadAssets();

			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>PerlinNoiseMovement</b></size>\nLogicBrick that applies Perlin Noise movement on an atom, relative to its parent position.\n\n</color>",
				255.0f, true
			);

			m_fScale = Utils.SetupSliderFloatWithRange(this, "Scale", 1, 0, 1, false);
			m_fSpeed = Utils.SetupSliderFloatWithRange(this, "Speed", 1, 0, 1, false);

            Utils.SetupSpacer(this, 20, false);
            Utils.SetupSpacer(this, 20, true);

			m_fDeltaPosX = Utils.SetupSliderFloatWithRange(this, "Delta Pos X", 0, -1, 1, false);
			m_fDeltaPosY = Utils.SetupSliderFloatWithRange(this, "Delta Pos Y", 0, -1, 1, false);
			m_fDeltaPosZ = Utils.SetupSliderFloatWithRange(this, "Delta Pos Z", 0, -1, 1, false);
			m_fDeltaRotX = Utils.SetupSliderFloatWithRange(this, "Delta Rot X", 0, -180, 180, true);
			m_fDeltaRotY = Utils.SetupSliderFloatWithRange(this, "Delta Rot Y", 0, -180, 180, true);
			m_fDeltaRotZ = Utils.SetupSliderFloatWithRange(this, "Delta Rot Z", 0, -180, 180, true);
		}


		private void FixedUpdate()
		{
            if (SuperController.singleton.freezeAnimation) return;
			
			if (!Mathf.Approximately(m_fSpeed.val, 0))
            {
				// move relative to parent
				Vector3    position = GetContainingAtom().parentAtom.mainController.transform.position;
				Quaternion rotation = GetContainingAtom().parentAtom.mainController.transform.rotation;
				// calculate deltas
			    m_fTime += Time.fixedDeltaTime * m_fSpeed.val;
				Vector3 deltaPos = Vector3.zero;
				deltaPos.x = GetRandomValue(0, m_fDeltaPosX.val);
				deltaPos.y = GetRandomValue(1, m_fDeltaPosY.val);
				deltaPos.z = GetRandomValue(2, m_fDeltaPosZ.val);
				Vector3 deltaRot = Vector3.zero;
				deltaRot.x = GetRandomValue(3, m_fDeltaRotX.val);
				deltaRot.y = GetRandomValue(4, m_fDeltaRotY.val);
				deltaRot.z = GetRandomValue(5, m_fDeltaRotZ.val);
				position += deltaPos;
				rotation *= Quaternion.Euler(deltaRot);
				GetMainController().transform.SetPositionAndRotation(position, rotation);
            }
		}

		private float GetRandomValue(int sector, float factor)
		{
			// Keep m_fTime within 10 "revolutions"
			if (m_fTime < -3600f) { m_fTime += 3600; }
			if (m_fTime > +3600f) { m_fTime -= 3600; }
			// Determine a 2D position on a "wobbling" circle around the origin, so there is some repeating randomness
			// Sector is [0...5] in 60 degree intervals, so we can easily generate several different roughly-similar-randomly behaving numbers
			float a = Mathf.Deg2Rad * ((360f / 6f * sector) + m_fTime);
			float r = 10 + 2 * Mathf.Sin(Mathf.Deg2Rad * m_fTime);
			float x = Mathf.Sin(a) * r;
			float y = Mathf.Cos(a) * r;
			// get perlin noise at that position
			return (2f * Mathf.PerlinNoise(x, y) - 1f) * factor * m_fScale.val;
		}
    }
}
