using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;
using MacGruber;

namespace HAL9001
{
    public class PerlinNoiseValue : MVRScript
    {
		private JSONStorableFloat m_fValueMin;
		private JSONStorableFloat m_fValueMax;
		private JSONStorableFloat m_fSpeed;
		private FloatTrigger      m_tValueOutTrigger;
        private float             m_fTime;


        public override void Init()
		{
			SimpleTriggerHandler.LoadAssets();

			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>PerlinNoiseValue</b></size>\nLogicBrick that generates values using a Perlin Noise function for smooth results.\n\n</color>",
				455.0f, true
			);

			m_tValueOutTrigger = new FloatTrigger(this, "OnValueSet");
			Utils.SetupButton(this, "Value Actions", m_tValueOutTrigger.OpenPanel, false);

			m_fValueMin = Utils.SetupSliderFloat(this, "Value Min", 0, 0, 1, false);
			m_fValueMax = Utils.SetupSliderFloat(this, "Value Max", 1, 0, 1, false);
			m_fSpeed    = Utils.SetupSliderFloatWithRange(this, "Speed",   1, 0, 1, false);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
		}

		private void OnAtomRename(string oldid, string newid)
		{
			m_tValueOutTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[m_tValueOutTrigger.Name] = m_tValueOutTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				m_tValueOutTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			m_tValueOutTrigger.Remove();
		}

		private void Update()
		{
            if (SuperController.singleton.freezeAnimation) return;
			
			if (m_fSpeed.val > 0)
            {
			    m_fTime += Time.deltaTime * m_fSpeed.val;
				// Keep m_fTime within 10 "revolutions"
				if (m_fTime < -3600f) { m_fTime += 3600; }
				if (m_fTime > +3600f) { m_fTime -= 3600; }
				// Determine a 2D position on a "wobbling" circle around the origin, so there is some repeating randomness
				float a = Mathf.Deg2Rad * m_fTime;
				float r = 10 + 2 * Mathf.Sin(Mathf.Deg2Rad * m_fTime);
				float x = Mathf.Sin(a) * r;
				float y = Mathf.Cos(a) * r;
				// get perlin noise at that position
				float v = Mathf.PerlinNoise(x, y);
                v = Mathf.Clamp01(v);
                v = m_fValueMin.val + v * (m_fValueMax.val - m_fValueMin.val);
                m_tValueOutTrigger.Trigger(v);
            }
            m_tValueOutTrigger.Update();
		}
    }
}
