﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;
using MacGruber;

namespace HAL9001
{
    public class ValueMapRelay : MVRScript
    {
		private JSONStorableFloat m_fValueIn;
		private JSONStorableFloat m_fInputFrom;
		private JSONStorableFloat m_fInputTo;
		private JSONStorableFloat m_fOutputFrom;
		private JSONStorableFloat m_fOutputTo;
		private JSONStorableBool  m_bExtrapolate;
		private FloatTrigger      m_tValueOutTrigger;

        public override void Init()
		{
			SimpleTriggerHandler.LoadAssets();

			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>ValueMapRelay</b></size>\nLogicBrick that relays a float value after mapping it from one range of values to another.\n\n</color>",
				455.0f, true
			);

			m_tValueOutTrigger = new FloatTrigger(this, "OnValueSet");
			Utils.SetupButton(this, "Value Actions", m_tValueOutTrigger.OpenPanel, false);

			m_fValueIn = Utils.SetupSliderFloat(this, "Value", 0, 0, 1, false);
			m_fValueIn.setCallbackFunction += Trigger;
			
			m_fInputFrom = Utils.SetupSliderFloatWithRange(this, "Input From", 0, 0, 1, false);
			m_fInputTo   = Utils.SetupSliderFloatWithRange(this, "Input To",   1, 0, 1, false);
			m_fInputFrom.setCallbackFunction += Trigger;
			m_fInputTo.setCallbackFunction   += Trigger;

			m_fOutputFrom = Utils.SetupSliderFloatWithRange(this, "Output From", 0, 0, 1, false);
			m_fOutputTo   = Utils.SetupSliderFloatWithRange(this, "Output To",   1, 0, 1, false);
			m_fOutputFrom.setCallbackFunction += Trigger;
			m_fOutputTo.setCallbackFunction   += Trigger;

			m_bExtrapolate = Utils.SetupToggle(this, "Extrapolate", false, false);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
		}

		private void OnAtomRename(string oldid, string newid)
		{
			m_tValueOutTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[m_tValueOutTrigger.Name] = m_tValueOutTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				m_tValueOutTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			m_tValueOutTrigger.Remove();
		}

		private void Update()
		{
			m_tValueOutTrigger.Update();
		}

		private void Trigger(float _)
		{
			if (this.enabled)
			{
				float v = (m_fValueIn.val - m_fInputFrom.val) / (m_fInputTo.val - m_fInputFrom.val);
				if (!m_bExtrapolate.val) 
				{
					if (v < 0) { v = 0; }
					if (v > 1) { v = 1; }
				}
				v = Mathf.Lerp(m_fOutputFrom.val, m_fOutputTo.val, v);
				m_tValueOutTrigger.Trigger(v);
			}
		}
    }
}
